unit VideoDevice;
{ Skeleton implementation of V++ video driver functions }

interface

uses
  Windows ;


type
  { Callback function definition }
  TVideoEvent = procedure( Id,Event:integer ) ; stdcall ;


{ Driver functions }
procedure OpenDriver ; stdcall ;
procedure CloseDriver ; stdcall ;
function  GetDriverName( Name:PChar; nChars:integer ) : PChar ; stdcall ;
function  GetDriverProvider( Name:PChar; nChars:integer ) : PChar ; stdcall ;
function  GetDriverVersion( Version:PChar; nChars:integer ) : PChar ; stdcall ;

{ Performance query functions }
function IsOsSupported : boolean ; stdcall ;
function GetMinTimeout : integer ; stdcall ;
function GetMaxFrameRate : integer ; stdcall ;

{ Device functions }
function  GetDeviceCount : integer ; stdcall ;
procedure SelectDevice( i:integer ) ; stdcall ;
function  SelectedDevice : integer ; stdcall ;
function  GetDeviceName( Name:PChar; nChars:integer ) : PChar ; stdcall ;

{ Status functions }
function DeviceReady : boolean ; stdcall ;
function DeviceError : integer ; stdcall ;
function DeviceMessage( Msg:PChar; nChars,Code:integer ) : PChar ; stdcall ;

{ Input channels }
function  GetChannelCount : integer ; stdcall ;
procedure SelectChannel( i:integer ) ; stdcall ;
function  SelectedChannel : integer ; stdcall ;

{ Basic acquisition }
procedure GetFrameInfo( var xSize,ySize,BitDepth,Samples:integer ) ; stdcall ;
procedure Snapshot ; stdcall ;
procedure ReadFrame( Data:pointer ) ; stdcall ;

{ Asynchronous capture }
procedure ASyncCapture ; stdcall ;
procedure ASyncStop ; stdcall ;
function  ASyncStatus : integer ; stdcall ;
procedure ASyncAbort ; stdcall ;

{ Frame buffer }
function  GetSampleFormat : integer ; stdcall ;
procedure GetPixelAspect( var xa,ya:integer ) ; stdcall ;
procedure ReadBuffer( Rect:TRect; Data:pointer ) ; stdcall ;
procedure WriteBuffer( Rect:TRect; Data:pointer ) ; stdcall ;

{ Region of interest }
procedure SetROI( Rect:TRect ) ; stdcall ;
function  GetROI : TRect ; stdcall ;
procedure ReadROI( Data:pointer ) ; stdcall ;

{ Continuous capture }
procedure SetContinuous( State:boolean ) ; stdcall ;
function  GetContinuous : boolean ; stdcall ;

{ Integration }
procedure SetExposureTime( msTime:integer ) ; stdcall ;
function  GetExposureTime : integer ; stdcall ;

{ Triggered mode }
procedure SetExposureMode( Mode:integer ) ; stdcall ;
function  GetExposureMode : integer ; stdcall ;
procedure SetTriggerTimeout( msTime:integer ) ; stdcall ;
function  GetTriggerTimeout : integer ; stdcall ;

{ Sequences }
procedure CaptureSequence( nFrames,msPeriod:integer; Data:pointer ) ; stdcall ;
function  CaptureSequenceEx( nFrames,msPeriod:integer; Data:pointer ) : integer ; stdcall ;

{ TTL lines }
procedure WriteTTL( Value:integer ) ; stdcall ;
function  ReadTTL : integer ; stdcall ;

{ User interface }
function ShowConfigForm( Parent:HWnd ) : integer ; stdcall ;
function ShowCustomForm( Parent:HWnd; Id:integer; Notify:TVideoEvent ) : boolean ; stdcall ;

{ VPascal interface }
function Execute( Cmd:PChar; Param:cardinal ) : integer ; stdcall ;



implementation

uses
  SysUtils ;


procedure OpenDriver ; stdcall ;
{ Initialize the driver and perform any hardware startup tasks }
{ This procedure is called when V++ loads the driver }

begin
end;  { OpenDriver }


procedure CloseDriver ; stdcall ;
{ Perform any clean up tasks required when V++ terminates or unloads the driver }

begin
end;  { CloseDriver }


function GetDriverName( Name:PChar; nChars:integer ) : PChar ; stdcall ;
{ Copy the driver's name into a string buffer }
{ Returns the PChar parameter as the function result }

begin
  StrCopy( Name,'Sample Video Driver' ) ;
  Result := Name ;
end;  { GetDriverName }


function GetDriverProvider( Name:PChar; nChars:integer ) : PChar ; stdcall ;
{ Copy the name of the driver provider into a string buffer }
{ Returns the PChar parameter as the function result }

begin
  StrCopy( Name,'Digital Optics' ) ;
  Result := Name ;
end;  { GetDriverProvider }


function GetDriverVersion( Version:PChar; nChars:integer ) : PChar ; stdcall ;
{ Copy a driver version string into a buffer }
{ Returns the PChar parameter as the function result }

begin
  StrCopy( Version,'1.0' ) ;
  Result := Version ;
end;  { GetDriverVersion }


function IsOsSupported : boolean ; stdcall ;
{ Return false if the running OS is not supported by the driver }

begin
  Result := true ;
end;  { IsOsSupported }


function GetMinTimeout : integer ; stdcall ;
{ Return the minimum capture timeout the host should use (ms) }

begin
  Result := 500 ;
end;  { GetMinTimeout }


function GetMaxFrameRate : integer ; stdcall ;
{ Return the maximum frame rate the host should expect (fps) }
{ The result of this function is not considered a guarantee of the rate }

begin
  Result := 30 ;
end;  { GetMaxFrameRate }


function GetDeviceCount : integer ; stdcall ;
{ Return the total number of video devices available using this driver }
{ This will usually be 1 }

begin
  Result := 1 ;
end;  { GetDeviceCount }


procedure SelectDevice( i:integer ) ; stdcall ;
{ Select the active video device }

begin
end;  { SelectDevice }


function SelectedDevice : integer ; stdcall ;
{ Return the index of the selected video device }

begin
  Result := 0 ;
end;  { SelectedDevice }


function GetDeviceName( Name:PChar; nChars:integer ) : PChar ; stdcall ;
{ Copy the name of the selected video device into a string buffer }
{ Returns the PChar parameter as the function result }

begin
  StrCopy( Name,'Sample Device' ) ;
  Result := Name ;
end;  { GetDeviceName }


function DeviceReady : boolean ; stdcall ;
{ Returns true if the selected video device is ready to digitize a frame }
{ If the device can't be detected or is busy then return false }

begin
  Result := true ;
end;  { DeviceReady }


function DeviceError : integer ; stdcall ;
{ Returns an error code for the selected video device }
{ ALWAYS return zero if there is no error condition }
{ If you also implement DeviceMessage() you can choose your own error codes }
{ Otherwise return 1 for a timeout and 2 for no device response }

begin
  Result := 0 ;
end;  { DeviceError }


function DeviceMessage( Msg:PChar; nChars,Code:integer ) : PChar ; stdcall ;
{ Copy an error message, for a given error code, into a string buffer }
{ The error codes are those returned by DeviceError }
{ Default messages will be used if you don't implement this function }

begin
  Result := Msg ;
end;  { DeviceMessage }


function GetChannelCount : integer ; stdcall ;
{ Returns the number of video input channels available on the selected device }

begin
  GetChannelCount := 1 ;
end;  { GetChannelCount }


procedure SelectChannel( i:integer ) ; stdcall ;
{ Select an input channel }

begin
end;  { SelectChannel }


function SelectedChannel : integer ; stdcall ;
{ Returns the selected video input channel }

begin
  Result := 0 ;
end;  { SelectedChannel }


procedure GetFrameInfo( var xSize,ySize,BitDepth,Samples:integer ) ; stdcall ;
{ Returns parameters of the full video frame available from the selected device }
{ Monochrome devices should return Samples = 1 }
{ RGB color devices (24-bit) should return Samples = 3 and BitDepth = 8 } 

begin
  xSize := 256 ;
  ySize := 256 ;
  BitDepth := 16 ;
  Samples := 1 ;
end;  { GetFrameInfo }


procedure Snapshot ; stdcall ;
{ Capture and store a single frame }
{ Do not return until the digitaization is complete }

begin
end;  { Snapshot }


procedure ReadFrame( Data:pointer ) ; stdcall ;
{ Copy the entire video frame into a block of memory }

begin
end;  { ReadFrame }


procedure ASyncCapture ; stdcall ;
{ Start a frame capture operation and return immediately }

begin
end;  { ASyncCapture }


procedure ASyncStop ; stdcall ;
{ Perform clean-up tasks following asynchronous capture }
{ Some frame grabbers require an explicit stop after starting a digitization }
{ If your hardware doesn't then you don't need to implement this routine. }

begin
end;  { ASyncStop }


function ASyncStatus : integer ; stdcall ;
{ Return the asynchronous capture status }
{ 0 = ready }
{ 1 = capture pending }
{ 2 = capture in progress }

begin
  Result := 0 ;
end;  { ASyncStatus }


procedure ASyncAbort ; stdcall ;
{ Terminate any pending capture operations }

begin
end;  { ASyncAbort }


procedure SetROI( Rect:TRect ) ; stdcall ;
{ Set a rectangular region of interest within the frame buffer }
{ The Rect parameter is interpreted as a Windows +1 rectangle }
{ Pass in an empty rectangle to cancel the region of interest }

begin
end;  { SetROI }


function GetROI : TRect ; stdcall ;
{ Return the current frame buffer region of interest }
{ An empty rectangle is returned if no ROI is selected }

begin
end;  { GetROI }


procedure ReadROI( Data:pointer ) ; stdcall ;
{ Copy image data from the region of interest into a buffer }
{ If no ROI is selected this routine should do nothing }

begin
end;  { ReadROI }


function GetSampleFormat : integer ; stdcall ;
{ Return the sample format of the pixels }
{ 0 = default }
{ 1 = unsigned integer }
{ 2 = signed integer }
{ 3 = floating point }
{ For color images this should define the component format }
{ For example, for standard RGB return 1 }

begin
  Result := 0 ;   // default
end;  { GetSampleFormat }


procedure GetPixelAspect( var xa,ya:integer ) ; stdcall ;
{ Return the aspect ratio of the pixels (usually 1:1) }

begin
  xa := 1 ;
  ya := 1 ;
end;  { GetPixelAspect }


procedure ReadBuffer( Rect:TRect; Data:pointer ) ; stdcall ;
{ Read a section from the frame buffer }
{ Rect is a +1 rectangle defined relative to the full video frame }
{ If Rect is an empty rectangle, read the entire frame }

begin
end;  { ReadBuffer }


procedure WriteBuffer( Rect:TRect; Data:pointer ) ; stdcall ;
{ Write data to a section of the frame buffer }
{ Rect is a +1 rectangle defined relative to the full video frame }
{ If Rect is an empty rectangle, write the entire frame }

begin
end;  { WriteBuffer }


procedure SetContinuous( State:boolean ) ; stdcall ;
{ Turn on/off continuous frame capture }

begin
end;  { SetContinuous }


function GetContinuous : boolean ; stdcall ;
{ Return the continuous capture state }

begin
  Result := false ;
end;  { GetContinuous }


procedure SetExposureTime( msTime:integer ) ; stdcall ;
{ Set the exposure time for an integrating camera }
{ Pass in zero to select the default exposure time }

begin
end;  { SetExposureTime }


function GetExposureTime : integer ; stdcall ;
{ Return the currently set exposure time }

begin
  Result := 0 ;
end;  { GetExposureTime }


procedure SetExposureMode( Mode:integer ) ; stdcall ;
{ Set the exposure mode to normal or triggered }

begin
end;  { SetExposureMode }


function GetExposureMode : integer ; stdcall ;
{ Return the current exposure mode }

begin
  Result := 0 ;
end;  { GetExposureMode }


procedure SetTriggerTimeout( msTime:integer ) ; stdcall ;
{ Set a timeout for triggered exposure }
{ Set the timeout to 0 to wait indefinitely }

begin
end;  { SetTriggerTimeout }


function GetTriggerTimeout : integer ; stdcall ;
{ Return the current trigger timeout value }

begin
  Result := 0 ;
end;  { GetTriggerTimeout }


procedure CaptureSequence( nFrames,msPeriod:integer; Data:pointer ) ; stdcall ;
{ Superceded by CaptureSequenceEx which returns the frame rate }

begin
  CaptureSequenceEx( nFrames,msPeriod,Data ) ;
end;  { CaptureSequence }


function CaptureSequenceEx( nFrames,msPeriod:integer; Data:pointer ) : integer ; stdcall ;
{ Capture a movie sequence into pre-allocated memory }
{ Returns the actual frame rate achieved }
{ Implement this function for devices with hardware sequence capture }
{ Set msPeriod=0 for a maximum rate sequence capture }
{ V++ provides sequence timing if this function is not implemented }

begin
  Result := 0 ;
end;  { CaptureSequenceEx }


procedure WriteTTL( Value:integer ) ; stdcall ;
{ Write to TLL outputs (up to 32 lines supported) }

begin
end;  { WriteTTL }


function ReadTTL : integer ; stdcall ;
{ Read state of TTL inputs }

begin
  Result := 0 ;
end;  { ReadTTL }


function ShowConfigForm( Parent:HWnd ) : integer ; stdcall ;
{ Display a modal hardware configuration dialog box }
{ Set the function result to the modal result of the dialog box }

begin
  Result := id_Cancel ;
end;  { ShowConfigForm }


function ShowCustomForm( Parent:HWnd; Id:integer; Notify:TVideoEvent ) : boolean ; stdcall ;
{ Display a custom hardware control form which may be modal or modeless }
{ This form may include live focus functions for devices that support it }
{ The callback function may be used to notify V++ that new data is available for readout }
{ When calling Notify, pass back Id unchanged as it is used by V++ to identify the driver }
{ Set the result to true to force a final readout on return from this procedure }
{ Avoid implementing hardware configuration here - do that in ShowConfigForm }

begin
  Result := false ;
end;  { ShowCustomForm }


function Execute( Cmd:PChar; Param:cardinal ) : integer ; stdcall ;
{ Execute a driver command }
{ Used to make specialized driver functions available in VPascal }
{ Interpretation of commands is entirely at the discretion of the driver }
{ Cmd is a string and Param is a 32-bit unsigned integer or pointer }
{ Result may or may not be meaningful depending on the driver }

begin
  Result := 0 ;
end;  { Execute }


end.
